/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 1997-2017 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://oss.oracle.com/licenses/CDDL+GPL-1.1
 * or LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */

/*
 * XSOMParserTest.java
 * JUnit based test
 *
 * Created on April 13, 2006, 9:54 AM
 */

import com.sun.xml.xsom.XSSchemaSet;
import com.sun.xml.xsom.parser.XSOMParser;
import com.sun.xml.xsom.parser.SchemaDocument;
import junit.framework.*;
import org.xml.sax.InputSource;

import java.net.URL;
import java.util.Set;

/**
 *
 * @author Farrukh S. Najmi
 */
public class XSOMParserTest extends TestCase {

    private static String docURLStr = "http://docs.oasis-open.org/regrep/v3.0/schema/lcm.xsd";
    //private static String docURLStr = "http://ebxmlrr.sourceforge.net/private/sun/irs/ContactMechanism/IRS-ContactMechanismCommonAggregateComponents-1.0.xsd";
    private static URL docURL = null;
    private static XSOMParser instance = null;

    public XSOMParserTest(String testName) {
        super(testName);
    }

    protected void setUp() throws Exception {
        if (docURL == null) {
            docURL = new URL(docURLStr);

            instance = new XSOMParser();
        }
    }

    protected void tearDown() throws Exception {
    }

    public static Test suite() {
        TestSuite suite = new TestSuite(XSOMParserTest.class);

        return suite;
    }

    /**
     * Test of parse method, of class com.sun.xml.xsom.parser.XSOMParser.
     */
    public void testParse() throws Exception {
        System.out.println("parse");

        //Following works.
        instance.parse(docURL);

        //Follwoing does not work
        InputSource inputSource = new InputSource(docURL.openStream());

        instance.parse(inputSource);
    }

    /**
     * Test of getDocuments method, of class com.sun.xml.xsom.parser.XSOMParser.
     */
    public void testGetDocuments() {
        System.out.println("getDocuments");


        Set<SchemaDocument> documents = instance.getDocuments();
        for (SchemaDocument doc : documents) {
            System.out.println("Schema document: "+doc.getSystemId());
            System.out.println("  target namespace: "+doc.getTargetNamespace());
            for (SchemaDocument ref : doc.getReferencedDocuments()) {
                System.out.print("    -> "+ref.getSystemId());
                if(doc.includes(ref))
                    System.out.print(" (include)");
                System.out.println();
            }
        }

    }

    /**
     * Test of getResult method, of class com.sun.xml.xsom.parser.XSOMParser.
     */
    public void testGetResult() throws Exception {
        System.out.println("getResult");

        XSSchemaSet result = instance.getResult();
    }


}
